/* -*- C -*- */
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

extern int __fsc_execvep(const char *path, const char *const *argv, const char *const *envp, int flage, int flagp);

/**
 * @fn int execv(const char *path, char *const *argv)
 * @brief execute a file.
 * @param path path of executable file.
 * @param argv arguments.
 * @return do not return on success , -1 on error.
 *
 * @crashcase invalid pointer, buffer overflow, non-terminated string
 * @fsctype string(ro), array(pointer)
 *
 * @author Lepidum Co., Ltd.
 */
int execv(const char *path, char *const *argv)
{
  return __fsc_execvep(path, argv, NULL, 0, 0);
}

/**
 * @fn int execve(const char *path, char *const *argv, char *const *envp)
 * @brief execute a file.
 * @param path path of executable file.
 * @param argv arguments.
 * @param envp environment variables.
 * @return do not return on success , -1 on error.
 *
 * @crashcase invalid pointer, buffer overflow, non-terminated string
 * @fsctype string(ro), array(pointer)
 *
 * @author Lepidum Co., Ltd.
 */
int execve(const char *path, char *const *argv, char *const *envp)
{
  return __fsc_execvep(path, argv, envp, 1, 0);
}

/**
 * @fn int execvp(const char *path, char *const *argv)
 * @brief execute a file.
 * @param path path of executable file.
 * @param argv arguments.
 * @return do not return on success , -1 on error.
 *
 * @crashcase invalid pointer, buffer overflow, non-terminated string
 * @fsctype string(ro), array(pointer)
 *
 * @author Lepidum Co., Ltd.
 */
int execvp(const char *path, char *const *argv)
{
  return __fsc_execvep(path, argv, NULL, 0, 1);
}

static int execlep(const char *path, const char *argv0, va_list va1, int flage, int flagp)
{
  int ret;
  va_list va2;
  va_copy(va2, va1);
  {
    size_t argc, i;
    const char **argv;
    char **envp = NULL;

    argc = 1;
    if (argv0 != NULL) {
      for (argc = 1; ; argc++) {
        char *v = va_arg(va1, char *);
        if (v == NULL) {
          if (flage) envp = va_arg(va1, char **); 
          break;
        }
      }
    } else {
      if (flage) envp = va_arg(va1, char **);
    }
    argv = malloc((argc + 1) * sizeof(char *));
    argv[0] = argv0;
    for (i = 1; i < argc; i++) {
      char *v = va_arg(va2, char *);
      argv[i] = v;
    }
    argv[i] = NULL;
    ret = __fsc_execvep(path, argv, envp, flage, flagp);
    free(argv);
  }
  va_end(va2);
  return ret;
}

/**
 * @fn int execl(const char *path, const char *arg0, ...)
 * @brief execute a file.
 * @param path path of executable file.
 * @param arg0 arguments.
 * @return do not return on success , -1 on error.
 *
 * @crashcase invalid pointer, buffer overflow, non-terminated string
 * @fsctype string(ro), array(pointer)
 *
 * @author Lepidum Co., Ltd.
 */
int execl(const char *path, const char *argv0, ...)
{
  int ret;
  va_list va;
  va_start(va, argv0);
  ret = execlep(path, argv0, va, 0, 0);
  va_end(va);
  return ret;
}

/**
 * @fn int execle(const char *path, const char *arg0, ...)
 * @brief execute a file.
 * @param path path of executable file.
 * @param arg0 arguments.
 * @return do not return on success , -1 on error.
 *
 * @crashcase invalid pointer, buffer overflow, non-terminated string
 * @fsctype string(ro), array(pointer)
 *
 * @author Lepidum Co., Ltd.
 */
int execle(const char *path, const char *argv0, ...)
{
  int ret;
  va_list va;
  va_start(va, argv0);
  ret = execlep(path, argv0, va, 1, 0);
  va_end(va);
  return ret;
}

/**
 * @fn int execlp(const char *path, const char *arg0, ...)
 * @brief execute a file.
 * @param path path of executable file.
 * @param arg0 arguments.
 * @return do not return on success , -1 on error.
 *
 * @crashcase invalid pointer, buffer overflow, non-terminated string
 * @fsctype string(ro), array(pointer)
 *
 * @author Lepidum Co., Ltd.
 */
int execlp(const char *path, const char *argv0, ...)
{
  int ret;
  va_list va;
  va_start(va, argv0);
  ret = execlep(path, argv0, va, 0, 1);
  va_end(va);
  return ret;
}
